package model

import (
	"database/sql"
	"database/sql/driver"
	"encoding/json"
	"fmt"
	"time"
)

type Date struct {
	time.Time
}

func NewDate(t time.Time) Date {
	loc, err := time.LoadLocation("Asia/Shanghai")
	if err != nil {
		panic(err)
	}
	t = t.In(loc)
	return Date{
		Time: time.Date(0, 1, 1, t.Hour(), t.Minute(), t.Second(), t.Nanosecond(), loc),
	}
}

var _ driver.Valuer = (*Date)(nil)

func (d Date) Value() (driver.Value, error) {
	loc, err := time.LoadLocation("Asia/Shanghai")
	if err != nil {
		panic(err)
	}
	return d.In(loc).Format("2006-01-02"), nil
}

var _ sql.Scanner = (*Date)(nil)

// Scan scans the time parsing it if necessary using timeFormat.
func (d *Date) Scan(src interface{}) (err error) {
	loc, err := time.LoadLocation("Asia/Shanghai")
	if err != nil {
		panic(err)
	}
	switch src := src.(type) {
	case time.Time:
		*d = NewDate(src)
		return nil
	case string:
		d.Time, err = time.ParseInLocation("2006-01-02", src, loc)
		return err
	case []byte:
		d.Time, err = time.ParseInLocation("2006-01-02", string(src), loc)
		return err
	case nil:
		d.Time = time.Time{}
		return nil
	default:
		return fmt.Errorf("unsupported data type: %T", src)
	}
}

var _ json.Marshaler = (*Date)(nil)

func (d Date) MarshalJSON() ([]byte, error) {
	return []byte(d.Time.Format("2006-01-02")), nil
}

var _ json.Unmarshaler = (*Date)(nil)

func (d *Date) UnmarshalJSON(raw []byte) error {
	loc, err := time.LoadLocation("Asia/Shanghai")
	if err != nil {
		return fmt.Errorf("unable to load time zone, %w", err)
	}
	s := string(raw)
	d.Time, err = time.ParseInLocation("2006-01-02", s, loc)
	return err
}
